@@ -20,9 +20,9 @@ import org.gradle.internal.service.scopes.Scope;
 import org.gradle.internal.service.scopes.ServiceScope;
 
 /**
- * Propagates notification that the build should be cancelled.
+ * Propagates notification that the build should be canceled.
  */
-@ServiceScope({Scope.Global.class, Scope.BuildSession.class, Scope.Build.class})
+@ServiceScope({Scope.Global.class, Scope.BuildSession.class})
 public interface BuildCancellationToken {
 
     boolean isCancellationRequested();
