@@ -26,6 +26,7 @@ import org.gradle.api.launcher.cli.WelcomeMessageConfiguration;
 import org.gradle.api.launcher.cli.WelcomeMessageDisplayMode;
 import org.gradle.api.logging.LogLevel;
 import org.gradle.api.logging.configuration.ConsoleOutput;
+import org.gradle.api.logging.configuration.ConsoleUnicodeSupport;
 import org.gradle.api.logging.configuration.LoggingConfiguration;
 import org.gradle.api.logging.configuration.ShowStacktrace;
 import org.gradle.api.logging.configuration.WarningMode;
@@ -145,6 +146,14 @@ public class StartParameter implements LoggingConfiguration, ParallelismConfigur
         return loggingConfiguration.getConsoleOutput();
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public ConsoleUnicodeSupport getConsoleUnicodeSupport() {
+        return loggingConfiguration.getConsoleUnicodeSupport();
+    }
+
     /**
      * {@inheritDoc}
      */
@@ -153,6 +162,14 @@ public class StartParameter implements LoggingConfiguration, ParallelismConfigur
         loggingConfiguration.setConsoleOutput(consoleOutput);
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @Override
+    public void setConsoleUnicodeSupport(ConsoleUnicodeSupport unicodeSupport) {
+        loggingConfiguration.setConsoleUnicodeSupport(unicodeSupport);
+    }
+
     /**
      * {@inheritDoc}
      */
@@ -250,6 +267,7 @@ public class StartParameter implements LoggingConfiguration, ParallelismConfigur
         p.gradleHomeDir = gradleHomeDir;
         p.setLogLevel(getLogLevel());
         p.setConsoleOutput(getConsoleOutput());
+        p.setConsoleUnicodeSupport(getConsoleUnicodeSupport());
         p.setShowStacktrace(getShowStacktrace());
         p.setWarningMode(getWarningMode());
         p.profile = profile;
@@ -619,7 +637,6 @@ public class StartParameter implements LoggingConfiguration, ParallelismConfigur
      *
      * @since 9.1.0
      */
-    @Incubating
     public boolean isTaskGraph() {
         return taskGraph;
     }
@@ -629,7 +646,6 @@ public class StartParameter implements LoggingConfiguration, ParallelismConfigur
      *
      * @since 9.1.0
      */
-    @Incubating
     public void setTaskGraph(boolean taskGraph) {
         this.taskGraph = taskGraph;
     }
