@@ -16,6 +16,8 @@
 
 package org.gradle.internal;
 
+import org.gradle.declarative.dsl.model.annotations.HiddenInDefinition;
+
 /**
  * Mutable value type for which mutation can be disabled at a certain point in time,
  * by calling the {@code disableFurtherMutations()} method.
@@ -23,6 +25,7 @@ package org.gradle.internal;
  * After {@code disableFurtherMutations()} has been called, any subsequent calls to methods that mutate
  * the value in any way will fail by throwing an {@code IllegalStateException}.
  */
+@HiddenInDefinition
 public interface FinalizableValue {
 
     /**
