@@ -37,6 +37,7 @@ import org.gradle.api.internal.file.collections.DirectoryFileTreeFactory;
 import org.gradle.api.internal.file.temp.TemporaryFileProvider;
 import org.gradle.api.internal.model.DefaultObjectFactory;
 import org.gradle.api.internal.model.NamedObjectInstantiator;
+import org.gradle.api.internal.plugins.PluginInspector;
 import org.gradle.api.internal.provider.PropertyFactory;
 import org.gradle.api.internal.tasks.TaskDependencyFactory;
 import org.gradle.api.internal.tasks.properties.annotations.AbstractOutputPropertyAnnotationHandler;
@@ -95,7 +96,6 @@ import org.gradle.internal.reflect.DirectInstantiator;
 import org.gradle.internal.reflect.Instantiator;
 import org.gradle.internal.scripts.DefaultScriptFileResolver;
 import org.gradle.internal.scripts.DefaultScriptFileResolverListeners;
-import org.gradle.internal.scripts.ScriptFileResolvedListener;
 import org.gradle.internal.scripts.ScriptFileResolver;
 import org.gradle.internal.scripts.ScriptFileResolverListeners;
 import org.gradle.internal.service.CachingServiceLocator;
@@ -140,8 +140,8 @@ public class GlobalScopeServices extends WorkerSharedGlobalScopeServices {
     private final GradleBuildEnvironment environment;
     private final AgentStatus agentStatus;
 
-    public GlobalScopeServices(final boolean longLiving, AgentStatus agentStatus, ClassPath additionalModuleClassPath, CurrentGradleInstallation currentGradleInstallation) {
-        super(additionalModuleClassPath, currentGradleInstallation);
+    public GlobalScopeServices(final boolean longLiving, AgentStatus agentStatus, CurrentGradleInstallation currentGradleInstallation) {
+        super(currentGradleInstallation);
         this.agentStatus = agentStatus;
         this.environment = () -> longLiving;
     }
@@ -149,13 +149,13 @@ public class GlobalScopeServices extends WorkerSharedGlobalScopeServices {
     @Override
     void configure(ServiceRegistration registration) {
         super.configure(registration);
-        registration.add(ScriptFileResolvedListener.class, ScriptFileResolverListeners.class, DefaultScriptFileResolverListeners.class);
+        registration.add(ScriptFileResolverListeners.class, DefaultScriptFileResolverListeners.class);
         registration.add(BuildLayoutFactory.class);
         registration.add(ValidateStep.ValidationWarningRecorder.class, WorkValidationWarningReporter.class, DefaultWorkValidationWarningRecorder.class);
     }
 
     @Provides
-    ScriptFileResolver createScriptFileResolver(ScriptFileResolvedListener listener) {
+    ScriptFileResolver createScriptFileResolver(ScriptFileResolverListeners listener) {
         return new DefaultScriptFileResolver(listener);
     }
 
@@ -390,4 +390,9 @@ public class GlobalScopeServices extends WorkerSharedGlobalScopeServices {
     ScriptSourceHasher createScriptSourceHasher() {
         return new DefaultScriptSourceHasher();
     }
+
+    @Provides
+    protected PluginInspector createPluginInspector(ModelRuleSourceDetector modelRuleSourceDetector) {
+        return new PluginInspector(modelRuleSourceDetector);
+    }
 }
