@@ -16,12 +16,9 @@
 
 package org.gradle.internal.declarativedsl
 
-import org.gradle.declarative.dsl.model.annotations.Adding
-import org.gradle.declarative.dsl.model.annotations.Builder
-import org.gradle.declarative.dsl.model.annotations.Configuring
-import org.gradle.declarative.dsl.model.annotations.HasDefaultValue
-import org.gradle.declarative.dsl.model.annotations.Restricted
+import org.gradle.declarative.dsl.model.annotations.HiddenInDefinition
 
-val hasDeclarativeAnnotation: (Annotation) -> Boolean = {
-    it is Builder || it is Configuring || it is Adding || it is Restricted || it is HasDefaultValue
+@Suppress("DEPRECATION")
+val hasHiddenInDefinitionAnnotation: (Annotation) -> Boolean = {
+    it is HiddenInDefinition
 }
