@@ -45,10 +45,17 @@ import org.gradle.internal.declarativedsl.analysis.ref
 import org.gradle.internal.declarativedsl.language.DataTypeInternal
 import org.gradle.internal.declarativedsl.schemaBuilder.SchemaBuildingContextElement.TagContextElement
 import org.gradle.internal.declarativedsl.schemaBuilder.SchemaBuildingTags.varargType
+import org.gradle.internal.declarativedsl.schemaBuilder.TypeDiscovery.DiscoveredClass.DiscoveryTag
+import org.gradle.internal.declarativedsl.schemaBuilder.TypeDiscovery.DiscoveredClass.DiscoveryTag.ContainerElement
+import org.gradle.internal.declarativedsl.schemaBuilder.TypeDiscovery.DiscoveredClass.DiscoveryTag.PropertyType
+import org.gradle.internal.declarativedsl.schemaBuilder.TypeDiscovery.DiscoveredClass.DiscoveryTag.Special
+import org.gradle.internal.declarativedsl.schemaBuilder.TypeDiscovery.DiscoveredClass.DiscoveryTag.Supertype
+import org.gradle.internal.declarativedsl.schemaBuilder.TypeDiscovery.DiscoveredClass.DiscoveryTag.UsedInMember
 import java.util.concurrent.atomic.AtomicLong
 import kotlin.reflect.KCallable
 import kotlin.reflect.KClass
 import kotlin.reflect.KFunction
+import kotlin.reflect.KParameter
 import kotlin.reflect.KType
 import kotlin.reflect.KTypeParameter
 import kotlin.reflect.KTypeProjection
@@ -60,10 +67,31 @@ import kotlin.reflect.typeOf
 interface SchemaBuildingHost {
     val topLevelReceiverClass: KClass<*>
 
+    fun classMembers(kClass: KClass<*>): ClassMembersForSchema
+    fun declarativeSupertypesHierarchy(kClass: KClass<*>): Iterable<MaybeDeclarativeClassInHierarchy>
+
+    /**
+     * Convert a [kClass] to a type that can be used as a DCL container (a block receiver).
+     */
     fun containerTypeRef(kClass: KClass<*>): DataTypeRef
+
+    /**
+     * Convert a [kType] to a type that can be used as a DCL model value type reference.
+     * Validates that the type is not nullable.
+     */
     fun modelTypeRef(kType: KType): DataTypeRef
+
+    /**
+     * Convert a [varargType] (element type) of a vararg function parameter to a DCL vararg (array) type reference.
+     */
     fun varargTypeRef(varargType: KType): DataTypeRef
 
+    /**
+     * Convert a [kType] to a DCL type reference without usage validation.
+     * The caller must validate the [kType] before using this function or using the returned type reference in the schema.
+     */
+    fun typeRef(kType: KType): DataTypeRef
+
     fun enterSchemaBuildingContext(contextElement: SchemaBuildingContextElement)
     fun leaveSchemaBuildingContext(contextElement: SchemaBuildingContextElement)
     val context: List<SchemaBuildingContextElement>
@@ -72,10 +100,10 @@ interface SchemaBuildingHost {
 fun <R> SchemaBuildingHost.inContextOfModelClass(kClass: KClass<*>, doBuildSchema: () -> R): R =
     inContextOf(SchemaBuildingContextElement.ModelClassContextElement(kClass), doBuildSchema)
 
-fun <R> SchemaBuildingHost.inContextOfModelMember(kCallable: KCallable<*>, doBuildSchema: () -> R): R =
+inline fun <R> SchemaBuildingHost.inContextOfModelMember(kCallable: KCallable<*>, doBuildSchema: () -> R): R =
     inContextOf(SchemaBuildingContextElement.ModelMemberContextElement(kCallable), doBuildSchema)
 
-fun <R> SchemaBuildingHost.withTag(tagContextElement: TagContextElement, doBuildSchema: () -> R): R =
+inline fun <R> SchemaBuildingHost.withTag(tagContextElement: TagContextElement, doBuildSchema: () -> R): R =
     inContextOf(tagContextElement, doBuildSchema)
 
 sealed interface SchemaBuildingContextElement {
@@ -99,14 +127,18 @@ sealed interface SchemaBuildingContextElement {
 
 object SchemaBuildingTags {
     fun parameter(name: String) = TagContextElement("parameter '$name'")
+    fun parameter(parameter: KParameter) = TagContextElement("parameter '${parameter.name ?: "(no name)"}'")
+    fun parameter(parameter: SupportedKParameter) = TagContextElement("parameter '${parameter.name ?: "(no name)"}'")
     fun receiverType(kClass: KClass<*>) = TagContextElement("receiver type '$kClass'")
     fun varargType(kType: KType) = TagContextElement("vararg type '$kType'")
     fun returnValueType(kType: KType) = TagContextElement("return value type '$kType'")
+    fun returnValueType(supportedType: SupportedTypeProjection.SupportedType) = TagContextElement("return value type '${supportedType.toKType()}'")
     fun externalObject(fqName: FqName) = TagContextElement("external object '${fqName.qualifiedName}'")
     fun namedDomainObjectContainer(name: String) = TagContextElement("nested named domain object container '$name'")
     fun elementTypeOfContainerSubtype(kClass: KClass<*>) = TagContextElement("element type of named domain object container subtype '$kClass'")
     fun typeArgument(argument: KTypeProjection) = TagContextElement("type argument '$argument'")
     fun configuredType(kType: KType) = TagContextElement("configured type '$kType'")
+    fun configuredType(supportedType: SupportedTypeProjection.SupportedType) = TagContextElement("configured type '${supportedType.toKType()}'")
 
     fun schemaClass(dataType: DataType.ClassDataType) = TagContextElement("schema type '${dataType.name}'")
     fun schemaFunction(function: SchemaFunction) = with(function) { TagContextElement("schema function '${simpleName}(${parameters.joinToString { "${it.name}: ${it.type}" }}): ${returnValueType}'") }
@@ -117,7 +149,7 @@ object SchemaBuildingTags {
     fun schemaTypeConstructor(constructor: DataConstructor) = TagContextElement("$constructor(${constructor.parameters.joinToString { "${it.name}: ${it.type}" }})")
 }
 
-private inline fun <R> SchemaBuildingHost.inContextOf(contextElement: SchemaBuildingContextElement, doBuildSchema: () -> R): R =
+inline fun <R> SchemaBuildingHost.inContextOf(contextElement: SchemaBuildingContextElement, doBuildSchema: () -> R): R =
     try {
         enterSchemaBuildingContext(contextElement)
         doBuildSchema()
@@ -147,6 +179,15 @@ class DataSchemaBuilder(
         override val context: List<SchemaBuildingContextElement>
             get() = currentContextStack
 
+        private val classMembersCache = mutableMapOf<KClass<*>, ClassMembersForSchema>()
+        private val declarativeSupertypesCache = mutableMapOf<KClass<*>, Iterable<MaybeDeclarativeClassInHierarchy>>()
+
+        override fun classMembers(kClass: KClass<*>): ClassMembersForSchema =
+            classMembersCache.getOrPut(kClass) { collectMembersForSchema(this, kClass) }
+
+        override fun declarativeSupertypesHierarchy(kClass: KClass<*>) =
+            declarativeSupertypesCache.getOrPut(kClass) { collectDeclarativeSuperclassHierarchy(kClass) }
+
         override fun containerTypeRef(kClass: KClass<*>): DataTypeRef {
             if (kClass.typeParameters.isNotEmpty()) {
                 schemaBuildingFailure("Cannot use the parameterized class '$kClass' as a configurable type")
@@ -154,8 +195,13 @@ class DataSchemaBuilder(
             return modelTypeRef(kClass.starProjectedType)
         }
 
-        override fun modelTypeRef(kType: KType): DataTypeRef =
-            typeRef(kType)
+        override fun modelTypeRef(kType: KType): DataTypeRef {
+            if (kType.isMarkedNullable) {
+                schemaBuildingFailure("Unsupported usage of a nullable type")
+            }
+
+            return typeRef(kType)
+        }
 
         override fun varargTypeRef(varargType: KType): DataTypeRef {
             val varargTypeSignature = typeSignatures.getOrPut(DefaultVarargSignature.name) { DefaultVarargSignature }
@@ -182,7 +228,7 @@ class DataSchemaBuilder(
             }
         }
 
-        private fun typeRef(kType: KType): DataTypeRef {
+        override fun typeRef(kType: KType): DataTypeRef {
             check(currentContextStack.isNotEmpty()) { "Cannot reference a type $kType outside of a context" }
 
             return when (val kClassifier = kType.classifier) {
@@ -229,7 +275,7 @@ class DataSchemaBuilder(
                         if (it.variance != KVariance.INVARIANT) {
                             schemaBuildingFailure("Illegal '${it.variance}' variance")
                         }
-                        val argumentTypeRef = this.typeRef(
+                        val argumentTypeRef = modelTypeRef(
                             it.type ?: schemaBuildingFailure("Type argument has no proper type")
                         )
                         TypeArgumentInternal.DefaultConcreteTypeArgument(argumentTypeRef)
@@ -421,10 +467,10 @@ class DataSchemaBuilder(
         val properties = mutableMapOf<KClass<*>, MutableMap<String, DataProperty>>()
 
         private
-        val propertyOriginalTypes = mutableMapOf<KClass<*>, MutableMap<String, KType>>()
+        val propertyOriginalTypes = mutableMapOf<KClass<*>, MutableMap<String, SupportedTypeProjection.SupportedType>>()
 
         private
-        val claimedFunctions = mutableMapOf<KClass<*>, MutableSet<KFunction<*>>>()
+        val claimedFunctions = mutableMapOf<KClass<*>, MutableSet<KCallable<*>>>()
 
         private val mutableSyntheticTypes = mutableMapOf<String, DataClass>()
 
@@ -436,12 +482,13 @@ class DataSchemaBuilder(
             propertyOriginalTypes.getOrPut(kClass) { mutableMapOf() }
         }
 
-        fun addProperty(kClass: KClass<*>, property: DataProperty, originalType: KType) {
+        fun addProperty(kClass: KClass<*>, property: DataProperty, originalType: SupportedTypeProjection.SupportedType) {
             properties.getOrPut(kClass) { mutableMapOf() }[property.name] = property
             propertyOriginalTypes.getOrPut(kClass) { mutableMapOf() }[property.name] = originalType
         }
 
-        fun claimFunction(kClass: KClass<*>, kFunction: KFunction<*>) {
+        // TODO replace with a generic schema member inclusion tracking mechanism
+        fun claimFunction(kClass: KClass<*>, kFunction: KCallable<*>) {
             claimedFunctions.getOrPut(kClass) { mutableSetOf() }.add(kFunction)
         }
 
@@ -455,10 +502,9 @@ class DataSchemaBuilder(
 
         fun getAllProperties(kClass: KClass<*>): List<DataProperty> = properties[kClass]?.values.orEmpty().toList()
 
-        fun getClaimedFunctions(kClass: KClass<*>): Set<KFunction<*>> = claimedFunctions[kClass].orEmpty()
+        fun getClaimedFunctions(kClass: KClass<*>): Set<KCallable<*>> = claimedFunctions[kClass].orEmpty()
 
         fun getProperty(kClass: KClass<*>, name: String) = properties[kClass]?.get(name)
-        fun getPropertyType(kClass: KClass<*>, name: String) = propertyOriginalTypes[kClass]?.get(name)
     }
 
     @Suppress("NestedBlockDepth")
@@ -471,15 +517,26 @@ class DataSchemaBuilder(
                 get() = host
         }
 
+        val allTypeDiscoveries: MutableSet<TypeDiscovery.DiscoveredClass> = mutableSetOf()
+
         val allTypesToVisit = buildSet {
             fun visit(type: KClass<*>) {
                 if (add(type)) {
-                    typeDiscovery.getClassesToVisitFrom(typeDiscoveryServices, type).forEach(::visit)
+                    val discoveriesToVisitNext = typeDiscovery.getClassesToVisitFrom(typeDiscoveryServices, type)
+                    allTypeDiscoveries.addAll(discoveriesToVisitNext)
+
+                    discoveriesToVisitNext.forEach {
+                        if (!it.isHidden) {
+                            visit(it.kClass)
+                        }
+                    }
                 }
             }
             types.forEach(::visit)
         }
 
+        checkDiscoveredTypeForIllegalHiddenTypeUsages(host, allTypeDiscoveries)
+
         return PreIndex().apply {
             allTypesToVisit.forEach { type ->
                 host.inContextOfModelClass(type) {
@@ -489,7 +546,7 @@ class DataSchemaBuilder(
                         it.claimedFunctions.forEach { f -> claimFunction(type, f) }
                         addProperty(
                             type,
-                            DefaultDataProperty(it.name, it.returnType, it.propertyMode, it.hasDefaultValue, it.isHiddenInDeclarativeDsl, it.isDirectAccessOnly),
+                            DefaultDataProperty(it.name, it.returnType, it.propertyMode, it.hasDefaultValue, it.isHiddenInDefinition, it.isDirectAccessOnly),
                             it.originalReturnType
                         )
                     }
@@ -498,6 +555,48 @@ class DataSchemaBuilder(
         }
     }
 
+    private fun checkDiscoveredTypeForIllegalHiddenTypeUsages(
+        host: SchemaBuildingHost,
+        allDiscoveries: MutableSet<TypeDiscovery.DiscoveredClass>
+    ) {
+        allDiscoveries.groupBy { it.kClass }.forEach { (kClass, discoveries) ->
+            if (!isIgnoredInVisibilityChecks(kClass) && discoveries.any { it.isHidden } && discoveries.any { !it.isHidden }) {
+                host.schemaBuildingFailure(
+                    "Type '${kClass.qualifiedName}' is a hidden type and cannot be directly used." +
+                            "\n  Appears as hidden:\n" +
+                            discoveries.filter { it.isHidden }
+                                .flatMap { it.discoveryTags }
+                                .joinToString("\n") { "    - ${discoveryTagDescription(it, kClass)}" } +
+                            "\n  Illegal usages:\n" +
+                            discoveries.filterNot { it.isHidden }
+                                .flatMap { it.discoveryTags }
+                                .filter { it !is Supertype || it.ofType != kClass } // Filter out the self appearance in the type hierarchy
+                                .joinToString("\n") { "    - ${discoveryTagDescription(it, kClass)}" }
+                )
+            }
+        }
+    }
+
+    /**
+     * Some types are widely used and do not make sense to hide; however, a model might accidentally hide them in a type hierarchy.
+     * Avoid reporting their usages in other types as errors.
+     */
+    private fun isIgnoredInVisibilityChecks(kClass: KClass<*>) = when (kClass) {
+        Iterable::class, Collection::class, List::class, Map::class, Set::class,
+        Any::class, Unit::class,
+        String::class, Int::class, Boolean::class, Long::class, Double::class -> true
+        else -> false
+    }
+
+    private fun discoveryTagDescription(tag: DiscoveryTag, inTypeHierarchyOf: KClass<*>): String = when (tag) {
+        is ContainerElement -> "as the element of a container '${tag.containerMember}'"
+        is PropertyType -> "as the property type of '${tag.kClass.qualifiedName}.${tag.propertyName}'"
+        is Supertype -> if (tag.ofType == inTypeHierarchyOf && tag.isHidden) "type '${inTypeHierarchyOf.qualifiedName}' is annotated as hidden" else
+            "in the supertypes of '${tag.ofType.qualifiedName}'"
+        is UsedInMember -> "referenced from member '${tag.member}'"
+        is Special -> tag.description
+    }
+
     @Suppress("UNCHECKED_CAST")
     private
     fun createDataType(
@@ -514,8 +613,7 @@ class DataSchemaBuilder(
             else -> {
                 val properties = preIndex.getAllProperties(kClass)
                 val functions = functionExtractor.memberFunctions(host, kClass, preIndex).toList()
-                val constructors = functionExtractor.constructors(host, kClass, preIndex).toList()
-                DefaultDataClass(kClass.fqName, kClass.java.name, listOf(), supertypesOf(kClass), properties, functions, constructors)
+                DefaultDataClass(kClass.fqName, kClass.java.name, listOf(), supertypesOf(kClass), properties, functions, emptyList())
                     .also { host.dataClassToKClass[it] = kClass }
             }
         }
