@@ -17,12 +17,8 @@
 package org.gradle.internal.declarativedsl.project
 
 import org.gradle.api.file.ProjectLayout
-import org.gradle.declarative.dsl.model.annotations.Restricted
 
 
 interface ProjectTopLevelReceiver {
-
-    @get:Restricted
     val layout: ProjectLayout
-    
 }
