@@ -16,12 +16,15 @@
 
 package org.gradle.kotlin.dsl.tooling.builders.internal
 
+import org.gradle.internal.buildtree.BuildModelParameters
 import org.gradle.internal.service.Provides
 import org.gradle.internal.service.ServiceRegistration
 import org.gradle.internal.service.ServiceRegistrationProvider
 import org.gradle.internal.service.scopes.AbstractGradleModuleServices
 import org.gradle.tooling.provider.model.ToolingModelBuilderRegistry
 import org.gradle.tooling.provider.model.internal.BuildScopeToolingModelBuilderRegistryAction
+import org.gradle.tooling.provider.model.internal.IntermediateToolingModelProvider
+import org.gradle.tooling.provider.model.internal.ToolingModelBuilderRegistrant
 
 class KotlinScriptingModelBuildersServices : AbstractGradleModuleServices() {
 
@@ -29,6 +32,10 @@ class KotlinScriptingModelBuildersServices : AbstractGradleModuleServices() {
         registration.addProvider(BuildScopeToolingServices)
     }
 
+    override fun registerProjectServices(registration: ServiceRegistration) {
+        registration.addProvider(ProjectScopeToolingServices)
+    }
+
     private object BuildScopeToolingServices : ServiceRegistrationProvider {
         @Provides
         fun createIdeBuildScopeToolingModelBuilderRegistryAction(): BuildScopeToolingModelBuilderRegistryAction =
@@ -39,4 +46,14 @@ class KotlinScriptingModelBuildersServices : AbstractGradleModuleServices() {
                 }
             }
     }
+
+    private object ProjectScopeToolingServices : ServiceRegistrationProvider {
+        @Provides
+        fun createToolingModelBuilderRegistrar(
+            modelParameters: BuildModelParameters,
+            intermediateModelProvider: IntermediateToolingModelProvider
+        ): ToolingModelBuilderRegistrant {
+            return KotlinScriptingModelBuildersRegistrant(modelParameters, intermediateModelProvider)
+        }
+    }
 }
