@@ -35,6 +35,7 @@ import org.gradle.cache.internal.locklistener.FileLockContentionHandler;
 import org.gradle.internal.UncheckedException;
 import org.gradle.internal.concurrent.CompositeStoppable;
 import org.gradle.internal.concurrent.Stoppable;
+import org.gradle.internal.os.OperatingSystem;
 import org.gradle.internal.time.ExponentialBackoff;
 import org.jspecify.annotations.Nullable;
 import org.slf4j.Logger;
@@ -42,10 +43,12 @@ import org.slf4j.LoggerFactory;
 
 import java.io.File;
 import java.io.IOException;
+import java.io.RandomAccessFile;
 import java.util.Locale;
 import java.util.Random;
 import java.util.Set;
 import java.util.concurrent.CopyOnWriteArraySet;
+import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.locks.Condition;
 import java.util.concurrent.locks.Lock;
 import java.util.concurrent.locks.ReentrantLock;
@@ -122,16 +125,48 @@ public class DefaultFileLockManager implements FileLockManager {
         }
         try {
             int port = fileLockContentionHandler.reservePort();
-            return new DefaultFileLock(canonicalTarget, options, targetDisplayName, operationDisplayName, port, whenContended);
+            return acquireFileLock(canonicalTarget, options, targetDisplayName, operationDisplayName, port, whenContended);
         } catch (Throwable t) {
             lockedFiles.remove(canonicalTarget);
             throw throwAsUncheckedException(t);
         }
     }
 
+    private DefaultFileLock acquireFileLock(
+        File canonicalTarget,
+        LockOptions options,
+        String targetDisplayName,
+        String operationDisplayName,
+        int port,
+        @Nullable Consumer<FileLockReleasedSignal> whenContended
+    ) throws Throwable {
+        if (options.isEnsureAcquiredLockRepresentsStateOnFileSystem()) {
+            DefaultFileLock fileLock = null;
+            while (fileLock == null) {
+                fileLock = new DefaultFileLock(canonicalTarget, options, targetDisplayName, operationDisplayName, port, whenContended);
+                // Verify the lock file hasn't been changed/deleted/recreated between opening a file handle and acquiring the lock.
+                if (!lockRepresentsStateOnFileSystem(fileLock)) {
+                    fileLock.close();
+                    fileLock = null;
+                }
+            }
+            return fileLock;
+        } else {
+            return new DefaultFileLock(canonicalTarget, options, targetDisplayName, operationDisplayName, port, whenContended);
+        }
+    }
+
+    private boolean lockRepresentsStateOnFileSystem(DefaultFileLock fileLock) {
+        // Skip for first lock access to optimize the first use case. We assume that if we hold the lock,
+        // and we are the first to access it, then no other Gradle process was able to delete/recreate the lock file.
+        return fileLock.isFirstLockAccess() || fileLock.lockRepresentsStateOnFileSystem();
+    }
+
     static File determineLockTargetFile(File target) {
         if (target.isDirectory()) {
             return new File(target, target.getName() + ".lock");
+        } else if (target.getName().endsWith(".lock")) {
+            return target;
         } else {
             return new File(target.getParentFile(), target.getName() + ".lock");
         }
@@ -143,11 +178,14 @@ public class DefaultFileLockManager implements FileLockManager {
         private final LockMode mode;
         private final String displayName;
         private final String operationDisplayName;
+        private final LockStateAccess lockStateAccess;
         private java.nio.channels.FileLock lock;
         private LockFileAccess lockFileAccess;
         private LockState lockState;
         private final int port;
         private final long lockId;
+        private final boolean isUseCrossVersionImplementation;
+        private final AtomicBoolean isFirstLockAccess = new AtomicBoolean(false);
 
         public DefaultFileLock(File target, LockOptions options, String displayName, String operationDisplayName, int port, @Nullable Consumer<FileLockReleasedSignal> whenContended) throws Throwable {
             this.port = port;
@@ -172,7 +210,8 @@ public class DefaultFileLockManager implements FileLockManager {
             }
 
             LockStateSerializer stateProtocol = options.isUseCrossVersionImplementation() ? new Version1LockStateSerializer() : new DefaultLockStateSerializer();
-            lockFileAccess = new LockFileAccess(lockFile, new LockStateAccess(stateProtocol));
+            this.lockStateAccess = new LockStateAccess(stateProtocol, () -> isFirstLockAccess.set(true));
+            lockFileAccess = new LockFileAccess(lockFile, lockStateAccess);
             try {
                 if (whenContended != null) {
                     fileLockContentionHandler.start(lockId, whenContended);
@@ -185,6 +224,7 @@ public class DefaultFileLockManager implements FileLockManager {
             }
 
             this.mode = lock.isShared() ? LockMode.Shared : LockMode.Exclusive;
+            this.isUseCrossVersionImplementation = options.isUseCrossVersionImplementation();
         }
 
         @Override
@@ -302,6 +342,33 @@ public class DefaultFileLockManager implements FileLockManager {
             return mode;
         }
 
+        private boolean lockRepresentsStateOnFileSystem() {
+            if (isUseCrossVersionImplementation || mode == LockMode.Shared) {
+                throw new UnsupportedOperationException("DefaultFileLock.lockRepresentsStateOnFileSystem() is not supported for shared or cross-version file locks.");
+            }
+
+            if (OperatingSystem.current().isWindows()) {
+                // On Windows, if we hold the lock it means lock was not updated on the file system,
+                // since OS won't allow to modify the state region or delete the file due to lock
+                return lock != null;
+            }
+
+            if (lock == null || !lockFile.exists()) {
+                return false;
+            }
+
+            // Compare the state directly from the file with the in-memory representation we have
+            try (RandomAccessFile randomAccessFile = new RandomAccessFile(lockFile, "r")) {
+                return !lockStateAccess.readState(randomAccessFile).hasBeenUpdatedSince(lockState);
+            } catch (IOException e) {
+                return false;
+            }
+        }
+
+        private boolean isFirstLockAccess() {
+            return isFirstLockAccess.get();
+        }
+
         /**
          * This method acquires a lock on the lock file.
          * <br><br>
