@@ -104,7 +104,7 @@ public class TestTreeModel {
      * @param stores the stores to load the models from
      * @return the merged tree model
      */
-    public static TestTreeModel loadModelFromStores(List<SerializableTestResultStore> stores) throws IOException {
+    public static TestTreeModel loadModelFromStores(List<SerializableTestResultStore> stores) throws Exception {
         Map<SmallPath, TestTreeModel.Builder> modelsByPath = new HashMap<>();
         int rootCount = stores.size();
         for (int i = 0; i < rootCount; i++) {
@@ -180,12 +180,22 @@ public class TestTreeModel {
 
             List<PerRootInfo.Builder> existingRootInfos = model.perRootInfoBuilders.get(rootIndex);
             if (!existingRootInfos.isEmpty()) {
-                // Only merge non-leaf nodes.  Leaf nodes might be repeated by test retries, so we'll want to add them all to the model.
+                // Only merge into non-leaf nodes. Leaf nodes might be repeated by test retries,
+                // so we'll want to add them all to the model if possible.
                 // The merging is necessary to support test engines like TestNG which can split test methods in a single class between
                 // multiple test workers.  These results must be recombined in the model to get the correct counts and report structure.
                 boolean isLeaf = rootInfo.isLeaf();
                 if (isLeaf) {
-                    existingRootInfos.add(rootInfo);
+                    // Merge into an existing non-leaf if possible, otherwise add as a new entry for retries.
+                    PerRootInfo.Builder existingNonLeaf = existingRootInfos.stream()
+                        .filter(info -> !info.isLeaf())
+                        .findFirst()
+                        .orElse(null);
+                    if (existingNonLeaf != null) {
+                        existingNonLeaf.merge(rootInfo);
+                    } else {
+                        existingRootInfos.add(rootInfo);
+                    }
                 } else {
                     // Merge into the one that is also not a leaf if possible, otherwise just merge into the first one.
                     PerRootInfo.Builder toMerge = existingRootInfos.stream()
