@@ -19,8 +19,8 @@ package org.gradle.api.internal.tasks.testing.detection;
 import org.gradle.api.file.FileTree;
 import org.gradle.api.internal.classpath.ModuleRegistry;
 import org.gradle.api.internal.tasks.testing.JvmTestExecutionSpec;
-import org.gradle.api.internal.tasks.testing.TestDefinitionProcessor;
 import org.gradle.api.internal.tasks.testing.TestDefinition;
+import org.gradle.api.internal.tasks.testing.TestDefinitionProcessor;
 import org.gradle.api.internal.tasks.testing.TestExecuter;
 import org.gradle.api.internal.tasks.testing.TestFramework;
 import org.gradle.api.internal.tasks.testing.TestResultProcessor;
@@ -42,6 +42,7 @@ import org.gradle.internal.actor.ActorFactory;
 import org.gradle.internal.time.Clock;
 import org.gradle.internal.work.WorkerLeaseService;
 import org.gradle.process.internal.worker.WorkerProcessFactory;
+import org.gradle.util.internal.IncubationLogger;
 
 import java.io.File;
 import java.util.ArrayList;
@@ -109,6 +110,15 @@ public class DefaultTestExecuter implements TestExecuter<JvmTestExecutionSpec> {
             testFrameworkDetector.setTestClasspath(classpath.getApplicationClasspath());
         }
 
+        /*
+         * We're possibly running non-class-based tests and there is a filter present.
+         *
+         * @see org.gradle.api.internal.tasks.testing.filter.FileTestSelectionMatcher
+         */
+        if (!testDefinitionDirs.isEmpty() && testFilter.hasPatterns()) {
+            IncubationLogger.incubatingFeatureUsed("Filtering non-class-based tests");
+        }
+
         TestDetector detector = new DefaultTestScanner(testClassFiles, testDefinitionDirs, testFramework.getDetector(), processor);
 
         // What is this?
